/*
AUTHOR:			Krissy Dyess (kdyess@powerpivotpro.com)
CREATED:		Jan-2018
PURPOSE:		Example SQL Code for Blog Post (Power Query Consecutive Days)
INSTRUCTIONS:	In SQL Server Management Studio, Connect to a Server, Right-Click Database & create a New Query, Select Execute or F5 to run the entire script
RESULTS:		Creates an example table with data and result is table with consecutive [create_date] through today's date
*/


-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
-- STEP 1:  CREATE a table structure for example data, using a temporary table called  (#StoreQuantity)
-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
	if OBJECT_ID('tempdb.dbo.#StoreQuantity', 'U') IS NOT NULL drop table #StoreQuantity

	create table #StoreQuantity
	(
	[store_id] nvarchar(20),
	[create_date] datetime,
	[qty] int
	)

-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
-- STEP 2:  INSERT example data into the temporary table structure
-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
	insert into #StoreQuantity select 'ABC123', cast(getdate()-160 as date), 10
	insert into #StoreQuantity select 'ABC123', cast(getdate()-159 as date), 11
	insert into #StoreQuantity select 'ABC123', cast(getdate()-158 as date), 12
	insert into #StoreQuantity select 'ABC123', cast(getdate()-157 as date), 13
	insert into #StoreQuantity select 'ABC123', cast(getdate()-156 as date), 14
	insert into #StoreQuantity select 'ABC123', cast(getdate()-155 as date), 15
	insert into #StoreQuantity select 'ABC123', cast(getdate()-150 as date), 20

	/*
	-- VIEW data to screen
		select * from #StoreQuantity order by create_date
	*/

-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
-- STEP 3:  CREATE a calendar date table with consecutive days, from 01/01/2017 to today
-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
	if OBJECT_ID('tempdb.dbo.#Dates', 'U') IS NOT NULL drop table #Dates

	select
	cast('A' as nvarchar(1)) as [join], -- Will be used to join 
	[dt],
	row_number() over (order by [dt]) as [dt_order]
	into #Dates
	from	
	-- Use DATEADD function to increment starting date by 1 day
	(
	select 
	dateadd(d,[incr],'2016-12-31') as [dt] 
	from
		-- Add an incremental indexing column 
		(
		select [incr] = row_number() over (order by [object_id])
		from
			-- Query the SQL system tables to get some data
			(
			select a.[object_id] from [sys].[all_columns] a group by a.[object_id]
			) as a
		) as b
	) as c
	where ([dt] <= getdate()) -- limits the results through to today's date
	
	/*
	-- VIEW data to screen
		select count(*) as [ttl_rows] from #Dates
		select * from #Dates order by [dt]
		select MIN([dt]) as [min_dt], MAX([dt]) as [min_dt], count(*) as [ttl_nbr_days] from #Dates
	*/

-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
-- STEP 4: JOIN consecutive calendar dates (#Dates) & store quantity table (#StoreQuantity), keep only records >= MIN([create_date])
-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
	if OBJECT_ID('tempdb.dbo.#StoreQuantityAllDays', 'U') IS NOT NULL drop table #StoreQuantityAllDays

	select
	sq.[store_id],
	d.[dt],
	row_number() over(partition by sq.[store_id] order by d.[dt] asc) as [dt_order]
	into #StoreQuantityAllDays
	from
		(
		select
		cast('A' as nvarchar(1)) as [join], -- Will be used to join 
		[store_id],
		min([create_date]) as [min_create_date]
		from #StoreQuantity
		group by 
		[store_id]
		) as sq
	Left Outer Join #Dates as d
	on (d.[join] = sq.[join])	-- Used to JOIN all date records to store quantity
	where d.[dt] >= sq.[min_create_date] -- Limit data to records >= store's first quantity date 

	/*
	-- VIEW data to screen
		select count(*) as [ttl_rows] from #StoreQuantityAllDays
		select * from #StoreQuantityAllDays order by [dt]
		select MIN([dt]) as [min_dt], MAX([dt]) as [min_dt], count(*) as [ttl_nbr_days] from #StoreQuantityAllDays
	*/

-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
-- STEP 5: JOIN to get the actual [qty] for days where there is a quantity value, all other days [qty] = 0
-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
	if OBJECT_ID('tempdb.dbo.#StoreQuantityFinal', 'U') IS NOT NULL drop table #StoreQuantityFinal

	select
	sqad.[store_id],
	sqad.[dt_order],
	sqad.[dt],
	sq.[create_date],
	sq.[qty]
	into #StoreQuantityFinal
	from #StoreQuantityAllDays as sqad Left Outer Join #StoreQuantity as sq 
	on (sqad.[store_id] = sq.[store_id])
	and (sqad.[dt] = sq.[create_date]) 

	/*
	-- VIEW data to screen
		select count(*) as [ttl_rows] from #StoreQuantityFinal
		select * from #StoreQuantityFinal order by [dt]
		select MIN([dt]) as [min_dt], MAX([dt]) as [min_dt], count(*) as [ttl_nbr_days] from #StoreQuantityFinal
	*/
	
	/*
	-- VIEW data to screen
		select * from #StoreQuantity order by [create_date]
		select [store_id], [dt] as [create_date], isnull([qty],0) as [qty] from #StoreQuantityFinal order by [dt]
	*/

	-- VIEW the INPUT example table & the FINAL result table to screen
		select * from #StoreQuantity
		select [store_id], [dt] as [create_date], isnull([qty],0) as [qty] from #StoreQuantityFinal order by [dt]
		select MIN([dt]) as [min_dt], MAX([dt]) as [min_dt], count(*) as [ttl_nbr_days] from #StoreQuantityFinal
